#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include <string.h>

#include "fat16.h"
#include "partition.h"
#include "sd_raw.h"

uint8_t find_file_in_dir(struct fat16_fs_struct* fs, struct fat16_dir_struct* dd, const char* name, struct fat16_dir_entry_struct* dir_entry);
struct fat16_file_struct* open_file_in_dir(struct fat16_fs_struct* fs, struct fat16_dir_struct* dd, const char* name);

int main( void )
{
  unsigned long int i;


/***** MCU-Takt **********************************************************/
  CLKPR = 0x80;			// Systemtakt halbieren
  CLKPR = 0x01;			// (Atmel-Datenblatt 7.12.2)

/***** Reset *************************************************************/
  DDRD = 0x80;
  PORTD &= 0x7F;
  for ( i=0; i<10000; i++ ) {
//    for ( j=0; j<1000; j++ ) {
//      PORTD &= 0x7F;
      PORTB ^= 0x0E;
//    }
  }
  PORTD |= 0x80;

/***** Port A PIO Daten **************************************************/
///  DDRA  = 0x00;
  PORTA = 0xFF;
  
/***** Port B ************************************************************/
  DDRB  = 0x0E;
  PORTB = 0x0E;

/***** Port C PIO Control ************************************************/
//  DDRC  = 0x0C;		// 00001100
//  PORTC = 0x04;		// 00001000
  
/***** Port D Input select ***********************************************/
  DDRD  |= 0x08;
//  PORTD  |= 0x7F;
  PORTD &= 0xF7;




/***** SD-Card initialisieren ********************************************/

  sd_raw_init();


#define BUFFER_LEN 250

  unsigned char buffer[BUFFER_LEN];







///// Versuch 1: Direktes Schreiben und Lesen /////////////////////////////

/*
// Schreiben auf SD-Karte
  for ( i=0; i<BUFFER_LEN; i++ )
    buffer[i] = i;
  sd_raw_write(100000,buffer,BUFFER_LEN);
  sd_raw_sync();
  for (;;) PORTB ^= 0x0E;
*/

/*
// Lesen von SD-Karte
  sd_raw_read(100000,buffer,BUFFER_LEN);
  for ( i=0; i<BUFFER_LEN; i++ ) {
    if ( buffer[i] != i ) for (;;);
  }
  for (;;) PORTB ^= 0x0E;
*/











///// Kartenzugriff aus Rolands Code //////////////////////////////////////

  /* open first partition */
  struct partition_struct* partition = partition_open(sd_raw_read,
                                                        sd_raw_read_interval,
                                                        sd_raw_write,
                                                        sd_raw_write_interval,
                                                        0
                                                       );

  if(!partition)
  {
    partition = partition_open(sd_raw_read,
                               sd_raw_read_interval,
                               sd_raw_write,
                               sd_raw_write_interval,
                               -1
                              );
    if(!partition)
    {
      return 1;
    }
  }

  /* open file system */
  struct fat16_fs_struct* fs = fat16_open(partition);
  if(!fs)
  {
    return 1;
  }

  /* open root directory */
  struct fat16_dir_entry_struct directory;
  fat16_get_dir_entry_of_path(fs, "/", &directory);

  struct fat16_dir_struct* dd = fat16_open_dir(fs, &directory);
  if(!dd)
  {
    return 1;
  }












///// Variablen fuer die folgenden Versuche ///////////////////////////////

  uint8_t result = 0;
  struct fat16_dir_entry_struct dir_entry;
  struct fat16_file_struct* fd;






///// Versuch 2: Dateien anlegen //////////////////////////////////////////


  if ( !fat16_create_file(dd,"datei.1",&dir_entry) ) result++;
  if ( !fat16_create_file(dd,"datei.2",&dir_entry) ) result++;
  if ( !fat16_create_file(dd,"datei.3",&dir_entry) ) result++;
  if ( !fat16_create_file(dd,"datei.4",&dir_entry) ) result++;
  if ( !fat16_create_file(dd,"datei.5",&dir_entry) ) result++;
  if ( !fat16_create_file(dd,"datei.6",&dir_entry) ) result++;
  if ( !fat16_create_file(dd,"datei.7",&dir_entry) ) result++;
  if ( !fat16_create_file(dd,"datei.8",&dir_entry) ) result++;
  if ( !fat16_create_file(dd,"datei.9",&dir_entry) ) result++;

  sd_raw_sync();

  if ( result == 0 )
    for (;;) {
      PORTB ^= 0x0E;
    }
  else if ( result == 1 )
    for (;;) {
      for ( i=0; i<2; i++ );
      PORTB ^= 0x0E;
    }
  else if ( result == 2 )
    for (;;) {
      for ( i=0; i<4; i++ );
      PORTB ^= 0x0E;
    }
  else if ( result == 3 )
    for (;;) {
      for ( i=0; i<6; i++ );
      PORTB ^= 0x0E;
    }
  else if ( result == 4 )
    for (;;) {
      for ( i=0; i<8; i++ );
      PORTB ^= 0x0E;
    }
  else if ( result == 5 )
    for (;;) {
      for ( i=0; i<10; i++ );
      PORTB ^= 0x0E;
    }
  else
    for (;;) {
    }












///// Versuch 3: in Datei schreiben / aus Datei lesen /////////////////////

/*
// Schreiben in Datei
  fd = open_file_in_dir(fs,dd,"datei.1");
  result = fat16_write_file(fd,"Hallole!",8);
//  result = fat16_resize_file(fd,1500);
  fat16_close_file(fd);
  sd_raw_sync();
  if ( result == 8 ) for (;;) PORTB ^= 0x0E;
  for (;;);
*/

/*
// Lesen aus Datei
  fd = open_file_in_dir(fs,dd,"datei.1");
  fat16_read_file(fd,buffer,8);
  fat16_close_file(fd);
  result = strncmp(buffer,"Hallole!",8);
  if ( result == 0 ) for (;;) PORTB ^= 0x0E;
  for (;;);
*/



  return 0;
}





///// Funktionen aus Rolands Code /////////////////////////////////////////

uint8_t find_file_in_dir(struct fat16_fs_struct* fs, struct fat16_dir_struct* dd, const char* name, struct fat16_dir_entry_struct* dir_entry)
{
    while(fat16_read_dir(dd, dir_entry))
    {
        if(strcmp(dir_entry->long_name, name) == 0)
        {
            fat16_reset_dir(dd);
            return 1;
        }
    }

    return 0;
}


struct fat16_file_struct* open_file_in_dir(struct fat16_fs_struct* fs, struct fat16_dir_struct* dd, const char* name)
{
    struct fat16_dir_entry_struct file_entry;
    if(!find_file_in_dir(fs, dd, name, &file_entry))
        return 0;

    return fat16_open_file(fs, &file_entry);
}

